==> rest-sdk-connector-model-0.5.0-sources.jar: com/mulesoft/connectivity/rest/sdk/internal/connectormodel/builder/translators/ConnectorOperationAdapterTranslator.java <==
/*
 * (c) 2003-2021 MuleSoft, Inc. This software is protected under international copyright
 * law. All use of this software is subject to MuleSoft's Master Subscription Agreement
 * (or other master license agreement) separately entered into in writing between you and
 * MuleSoft. If such an agreement is not in place, you may not use the software.
 */
package com.mulesoft.connectivity.rest.sdk.internal.connectormodel.builder.translators;

import static com.mulesoft.connectivity.rest.sdk.internal.connectormodel.parameter.ParameterType.BODY;
import static com.mulesoft.connectivity.rest.sdk.internal.connectormodel.parameter.ParameterType.HEADER;
import static com.mulesoft.connectivity.rest.sdk.internal.connectormodel.parameter.ParameterType.QUERY;
import static com.mulesoft.connectivity.rest.sdk.internal.connectormodel.parameter.ParameterType.URI;
import static java.util.Optional.ofNullable;
import static org.apache.commons.lang3.StringUtils.isEmpty;

import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.builder.ConnectorModelBuilder;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.builder.OperationBuilder;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.builder.ParameterBuilder;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.builder.TriggerParameterBindingBuilder;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.builder.TriggerParameterBuilder;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.builder.TypeDefinitionBuilder;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.generic.ParameterDataType;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.parameter.ParameterType;
import com.mulesoft.connectivity.rest.sdk.internal.descriptor.model.ConnectorDescriptor;
import com.mulesoft.connectivity.rest.sdk.internal.descriptor.model.DataTypeDescriptor;
import com.mulesoft.connectivity.rest.sdk.internal.descriptor.model.OperationAdapterDescriptor;
import com.mulesoft.connectivity.rest.sdk.internal.descriptor.model.TriggerParameterBindingDescriptor;
import com.mulesoft.connectivity.rest.sdk.internal.descriptor.model.TriggerParameterBindingsDescriptor;
import com.mulesoft.connectivity.rest.sdk.internal.descriptor.model.TriggerParameterDescriptor;

import java.util.LinkedList;
import java.util.List;
import java.util.Optional;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import javax.ws.rs.core.MediaType;

import org.apache.commons.lang3.StringUtils;

public class ConnectorOperationAdapterTranslator {

  private static final ConnectorOperationAdapterTranslator INSTANCE = new ConnectorOperationAdapterTranslator();

  public static void translateOperationAdapters(ConnectorDescriptor connectorDescriptor,
                                                ConnectorModelBuilder connectorModelBuilder) {
    INSTANCE._translateOperationAdapters(connectorDescriptor, connectorModelBuilder);
  }

  private void _translateOperationAdapters(ConnectorDescriptor connectorDescriptor, ConnectorModelBuilder connectorModelBuilder) {
    for (OperationAdapterDescriptor operationAdapterDescriptor : connectorDescriptor.getOperationAdapterDescriptors()) {
      buildOperationAdapter(operationAdapterDescriptor, connectorModelBuilder);
    }
  }

  private void buildOperationAdapter(OperationAdapterDescriptor operationAdapterDescriptor,
                                     ConnectorModelBuilder connectorModelBuilder) {
    OperationBuilder baseOperationBuilder = resolveBaseOperationBuilder(operationAdapterDescriptor, connectorModelBuilder);
    final OperationBuilder operationBuilder =
        connectorModelBuilder.addOperationBuilder(operationAdapterDescriptor.getOperationId(),
                                                  baseOperationBuilder.getPath(), baseOperationBuilder.getMethod());
    buildOperationAdapter(operationBuilder, operationAdapterDescriptor, baseOperationBuilder);
  }

  private OperationBuilder resolveBaseOperationBuilder(OperationAdapterDescriptor operationAdapterDescriptor,
                                                       ConnectorModelBuilder connectorModelBuilder) {
    final String baseOperationId = operationAdapterDescriptor.getBaseOperationId();
    final OperationBuilder operationBuildersByOperationId =
        connectorModelBuilder.getOperationBuildersByOperationId(baseOperationId)
            .orElseThrow(() -> new IllegalArgumentException(String.format("Base operation not found %s", baseOperationId)));
    if (!operationBuildersByOperationId.isAdapter()) {
      return operationBuildersByOperationId;
    } else {
      throw new IllegalArgumentException(String.format("Base operation %s is not an endpoint operation.", baseOperationId));
    }
  }

  private void buildOperationAdapter(OperationBuilder operationBuilder, OperationAdapterDescriptor operationAdapterDescriptor,
                                     OperationBuilder prototypeOperation) {

    operationBuilder.apply(prototypeOperation);
    operationBuilder.ignored(false);
    ofNullable(operationAdapterDescriptor.getDisplayName()).ifPresent(operationBuilder::displayName);
    ofNullable(operationAdapterDescriptor.getDescription()).ifPresent(operationBuilder::description);
    final List<TriggerParameterBindingBuilder> requestBindings =
        buildRequestBindings(operationAdapterDescriptor.getRequestBindings());
    operationBuilder.requestBindings(requestBindings);
    final List<TriggerParameterBindingBuilder> responseBindings =
        buildResponseBindings(operationAdapterDescriptor.getResponseBindings());
    operationBuilder.responseBindings(responseBindings);
    operationBuilder.uriParameters(filterParametersAlreadyBound(operationBuilder.getUriParameters(), requestBindings));
    operationBuilder.queryParameters(filterParametersAlreadyBound(operationBuilder.getQueryParameters(), requestBindings));
    operationBuilder.headers(filterParametersAlreadyBound(operationBuilder.getHeaders(), requestBindings));
    final List<TriggerParameterDescriptor> parameters = operationAdapterDescriptor.getParameters();
    operationBuilder.parameters(buildParameters(parameters));

    // TODO Signaling no content parameter exposed with this coupling condition
    getBodyBinding(requestBindings).ifPresent(triggerParameterBindingBuilder -> {
      operationBuilder.getInputMetadata().clear();
    });

    getBodyBinding(responseBindings).ifPresent(triggerParameterBindingBuilder -> {
      final TypeDefinitionBuilder typeDefinitionBuilder = triggerParameterBindingBuilder.getTypeDefinitionBuilder();
      MediaType mediaType = typeDefinitionBuilder.getMediaType();
      final Supplier<String> rawSchemaSupplier = typeDefinitionBuilder.getRawSchemaSupplier();
      if (mediaType != null && rawSchemaSupplier != null) {
        // TODO: Review
        operationBuilder
            .forceOutputTypeSchema(rawSchemaSupplier.get())
            .setDefaultOutputMediaType(mediaType);
        // operationBuilder.getOutputMetadataBuilder(mediaType).typeSchema();
      }
    });
    operationBuilder.adapter();
    if (ofNullable(operationAdapterDescriptor.getSidecarOperation()).orElse(false)) {
      operationBuilder.sidecar();
    }
  }

  private List<ParameterBuilder> filterParametersAlreadyBound(List<ParameterBuilder> parameters,
                                                              List<TriggerParameterBindingBuilder> requestBindings) {
    if (requestBindings == null) {
      return parameters;
    }
    return parameters.stream().filter(parameter -> !isParameterAlreadyBound(parameter, requestBindings))
        .collect(Collectors.toList());
  }

  private boolean isParameterAlreadyBound(ParameterBuilder parameter, List<TriggerParameterBindingBuilder> requestBindings) {
    if (parameter == null) {
      throw new IllegalArgumentException();
    }
    if (requestBindings == null) {
      return false;
    }

    return requestBindings.stream().anyMatch(parameterBinding -> isSameBinding(parameter, parameterBinding));
  }

  private Optional<TriggerParameterBindingBuilder> getBodyBinding(List<TriggerParameterBindingBuilder> requestBindings) {
    if (requestBindings == null) {
      return Optional.empty();
    }

    return requestBindings.stream().filter(this::isBodyBinding).findFirst();
  }

  private boolean isSameBinding(ParameterBuilder parameter, TriggerParameterBindingBuilder parameterBinding) {
    return parameter.getParameterType().equals(parameterBinding.getParameterType())
        && parameter.getExternalName().equals(parameterBinding.getName());
  }

  private boolean isBodyBinding(TriggerParameterBindingBuilder parameterBinding) {
    return parameterBinding.getParameterType().equals(BODY);
  }

  private List<TriggerParameterBindingBuilder> buildResponseBindings(TriggerParameterBindingsDescriptor parameterBindings) {
    return buildRequestBindings(parameterBindings);
  }

  private List<TriggerParameterBindingBuilder> buildRequestBindings(TriggerParameterBindingsDescriptor parameterBindings) {
    if (parameterBindings == null) {
      return null;
    }

    final List<TriggerParameterBindingBuilder> parameterBindingList = new LinkedList<>();

    if (parameterBindings.getUriParameters() != null) {
      for (TriggerParameterBindingDescriptor uriParam : parameterBindings.getUriParameters()) {
        parameterBindingList.add(buildParameterBinding(uriParam, URI));
      }
    }

    if (parameterBindings.getQueryParameters() != null) {
      for (TriggerParameterBindingDescriptor queryParam : parameterBindings.getQueryParameters()) {
        parameterBindingList.add(buildParameterBinding(queryParam, QUERY));
      }
    }

    if (parameterBindings.getHeaders() != null) {
      for (TriggerParameterBindingDescriptor header : parameterBindings.getHeaders()) {
        parameterBindingList.add(buildParameterBinding(header, HEADER));
      }
    }

    if (parameterBindings.getBodyBindings() != null) {
      for (TriggerParameterBindingDescriptor body : parameterBindings.getBodyBindings()) {
        parameterBindingList.add(buildParameterBinding(body, BODY));
      }
    }

    final String requestBodyExpression = parameterBindings.getRequestBodyExpression();
    if (requestBodyExpression != null) {
      parameterBindingList.add(buildBodyParameterBinding(requestBodyExpression));
    }

    return parameterBindingList;
  }

  private TriggerParameterBindingBuilder buildBodyParameterBinding(String value) {
    TriggerParameterBindingBuilder triggerParameterBindingBuilder = new TriggerParameterBindingBuilder(BODY.getName(), BODY);
    triggerParameterBindingBuilder.value(value);
    return triggerParameterBindingBuilder;
  }

  private TriggerParameterBindingBuilder buildParameterBinding(TriggerParameterBindingDescriptor descriptor,
                                                               ParameterType parameterType) {
    TriggerParameterBindingBuilder triggerParameterBindingBuilder =
        new TriggerParameterBindingBuilder(descriptor.getName(), parameterType);
    triggerParameterBindingBuilder.value(descriptor.getValue());

    final String schemaType = descriptor.getInputType();
    if (StringUtils.isNotBlank(schemaType)) {
      final TypeDefinitionBuilder typeDefinitionBuilder = triggerParameterBindingBuilder.getTypeDefinitionBuilder();
      typeDefinitionBuilder.object().typeSchema(descriptor::getInputType);
      typeDefinitionBuilder.mediaType(loadMediaType(descriptor.getContentType()));
    }

    return triggerParameterBindingBuilder;
  }

  private List<TriggerParameterBuilder> buildParameters(List<TriggerParameterDescriptor> parameterDescriptors) {
    final List<TriggerParameterBuilder> parameters = new LinkedList<>();
    if (parameterDescriptors != null) {
      for (TriggerParameterDescriptor parameterDescriptor : parameterDescriptors) {
        parameters.add(buildParameter(parameterDescriptor));
      }
    }

    return parameters;
  }

  private TriggerParameterBuilder buildParameter(TriggerParameterDescriptor parameterDescriptor) {
    TriggerParameterBuilder triggerParameterBuilder = new TriggerParameterBuilder(parameterDescriptor.getName());
    triggerParameterBuilder
        .displayName(parameterDescriptor.getDisplayName())
        .description(parameterDescriptor.getDescription())
        .required(parameterDescriptor.isRequired());

    final DataTypeDescriptor dataType = parameterDescriptor.getDataType();
    if (dataType != null) {
      final ParameterDataType parameterDataType = ParameterDataType.forName(dataType.getName());
      triggerParameterBuilder.dataType(parameterDataType);
    } else {
      final TypeDefinitionBuilder typeDefinitionBuilder = triggerParameterBuilder.getTypeDefinitionBuilder();
      if (StringUtils.isNotBlank(parameterDescriptor.getInputType())) {
        typeDefinitionBuilder.object().typeSchema(parameterDescriptor::getInputType);
      }
      typeDefinitionBuilder.mediaType(loadMediaType(parameterDescriptor.getContentType()));
    }

    return triggerParameterBuilder;
  }

  private static MediaType loadMediaType(String contentType) {
    return isEmpty(contentType) ? null : MediaType.valueOf(contentType);
  }

}

==> rest-sdk-connector-model-0.6.0-sources.jar: com/mulesoft/connectivity/rest/sdk/internal/connectormodel/builder/translators/ConnectorOperationAdapterTranslator.java <==
/*
 * (c) 2003-2021 MuleSoft, Inc. This software is protected under international copyright
 * law. All use of this software is subject to MuleSoft's Master Subscription Agreement
 * (or other master license agreement) separately entered into in writing between you and
 * MuleSoft. If such an agreement is not in place, you may not use the software.
 */
package com.mulesoft.connectivity.rest.sdk.internal.connectormodel.builder.translators;

import static com.mulesoft.connectivity.rest.sdk.internal.connectormodel.parameter.ParameterType.BODY;
import static com.mulesoft.connectivity.rest.sdk.internal.connectormodel.parameter.ParameterType.HEADER;
import static com.mulesoft.connectivity.rest.sdk.internal.connectormodel.parameter.ParameterType.QUERY;
import static com.mulesoft.connectivity.rest.sdk.internal.connectormodel.parameter.ParameterType.URI;
import static java.util.Optional.ofNullable;
import static org.apache.commons.lang3.StringUtils.isEmpty;

import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.builder.AuxiliarParameterBindingBuilder;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.builder.AuxiliarParameterBuilder;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.builder.ConnectorModelBuilder;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.builder.OperationBuilder;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.builder.ParameterBuilder;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.builder.TypeDefinitionBuilder;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.generic.ParameterDataType;
import com.mulesoft.connectivity.rest.sdk.internal.connectormodel.parameter.ParameterType;
import com.mulesoft.connectivity.rest.sdk.internal.descriptor.model.AuxiliarParameterBindingDescriptor;
import com.mulesoft.connectivity.rest.sdk.internal.descriptor.model.AuxiliarParameterBindingsDescriptor;
import com.mulesoft.connectivity.rest.sdk.internal.descriptor.model.AuxiliarParameterDescriptor;
import com.mulesoft.connectivity.rest.sdk.internal.descriptor.model.ConnectorDescriptor;
import com.mulesoft.connectivity.rest.sdk.internal.descriptor.model.DataTypeDescriptor;
import com.mulesoft.connectivity.rest.sdk.internal.descriptor.model.OperationAdapterDescriptor;

import java.util.LinkedList;
import java.util.List;
import java.util.Optional;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import javax.ws.rs.core.MediaType;

import org.apache.commons.lang3.StringUtils;

public class ConnectorOperationAdapterTranslator {

  private static final ConnectorOperationAdapterTranslator INSTANCE = new ConnectorOperationAdapterTranslator();

  public static void translateOperationAdapters(ConnectorDescriptor connectorDescriptor,
                                                ConnectorModelBuilder connectorModelBuilder) {
    INSTANCE._translateOperationAdapters(connectorDescriptor, connectorModelBuilder);
  }

  private void _translateOperationAdapters(ConnectorDescriptor connectorDescriptor, ConnectorModelBuilder connectorModelBuilder) {
    for (OperationAdapterDescriptor operationAdapterDescriptor : connectorDescriptor.getOperationAdapterDescriptors()) {
      buildOperationAdapter(operationAdapterDescriptor, connectorModelBuilder);
    }
  }

  private void buildOperationAdapter(OperationAdapterDescriptor operationAdapterDescriptor,
                                     ConnectorModelBuilder connectorModelBuilder) {
    OperationBuilder baseOperationBuilder = resolveBaseOperationBuilder(operationAdapterDescriptor, connectorModelBuilder);
    final OperationBuilder operationBuilder =
        connectorModelBuilder.addOperationBuilder(operationAdapterDescriptor.getOperationId(),
                                                  baseOperationBuilder.getPath(), baseOperationBuilder.getMethod());
    buildOperationAdapter(operationBuilder, operationAdapterDescriptor, baseOperationBuilder);
  }

  private OperationBuilder resolveBaseOperationBuilder(OperationAdapterDescriptor operationAdapterDescriptor,
                                                       ConnectorModelBuilder connectorModelBuilder) {
    final String baseOperationId = operationAdapterDescriptor.getBaseOperationId();
    final OperationBuilder operationBuildersByOperationId =
        connectorModelBuilder.getOperationBuildersByOperationId(baseOperationId)
            .orElseThrow(() -> new IllegalArgumentException(String.format("Base operation not found %s", baseOperationId)));
    if (!operationBuildersByOperationId.isAdapter()) {
      return operationBuildersByOperationId;
    } else {
      throw new IllegalArgumentException(String.format("Base operation %s is not an endpoint operation.", baseOperationId));
    }
  }

  private void buildOperationAdapter(OperationBuilder operationBuilder, OperationAdapterDescriptor operationAdapterDescriptor,
                                     OperationBuilder prototypeOperation) {

    operationBuilder.apply(prototypeOperation);
    operationBuilder.ignored(false);
    ofNullable(operationAdapterDescriptor.getDisplayName()).ifPresent(operationBuilder::displayName);
    ofNullable(operationAdapterDescriptor.getDescription()).ifPresent(operationBuilder::description);
    final List<AuxiliarParameterBindingBuilder> requestBindings =
        buildRequestBindings(operationAdapterDescriptor.getRequestBindings());
    operationBuilder.requestBindings(requestBindings);
    final List<AuxiliarParameterBindingBuilder> responseBindings =
        buildResponseBindings(operationAdapterDescriptor.getResponseBindings());
    operationBuilder.responseBindings(responseBindings);
    operationBuilder.uriParameters(filterParametersAlreadyBound(operationBuilder.getUriParameters(), requestBindings));
    operationBuilder.queryParameters(filterParametersAlreadyBound(operationBuilder.getQueryParameters(), requestBindings));
    operationBuilder.headers(filterParametersAlreadyBound(operationBuilder.getHeaders(), requestBindings));
    final List<AuxiliarParameterDescriptor> parameters = operationAdapterDescriptor.getParameters();
    operationBuilder.parameters(buildParameters(parameters));

    // TODO Signaling no content parameter exposed with this coupling condition
    getBodyBinding(requestBindings).ifPresent(auxiliarParameterBindingBuilder -> {
      operationBuilder.getInputMetadata().clear();
    });

    getBodyBinding(responseBindings).ifPresent(auxiliarParameterBindingBuilder -> {
      final TypeDefinitionBuilder typeDefinitionBuilder = auxiliarParameterBindingBuilder.getTypeDefinitionBuilder();
      MediaType mediaType = typeDefinitionBuilder.getMediaType();
      final Supplier<String> rawSchemaSupplier = typeDefinitionBuilder.getRawSchemaSupplier();
      if (mediaType != null && rawSchemaSupplier != null) {
        // TODO: Review
        operationBuilder
            .forceOutputTypeSchema(rawSchemaSupplier.get())
            .setDefaultOutputMediaType(mediaType);
        // operationBuilder.getOutputMetadataBuilder(mediaType).typeSchema();
      }
    });
    operationBuilder.adapter();
    if (ofNullable(operationAdapterDescriptor.getSidecarOperation()).orElse(false)) {
      operationBuilder.sidecar();
    }
  }

  private List<ParameterBuilder> filterParametersAlreadyBound(List<ParameterBuilder> parameters,
                                                              List<AuxiliarParameterBindingBuilder> requestBindings) {
    if (requestBindings == null) {
      return parameters;
    }
    return parameters.stream().filter(parameter -> !isParameterAlreadyBound(parameter, requestBindings))
        .collect(Collectors.toList());
  }

  private boolean isParameterAlreadyBound(ParameterBuilder parameter, List<AuxiliarParameterBindingBuilder> requestBindings) {
    if (parameter == null) {
      throw new IllegalArgumentException();
    }
    if (requestBindings == null) {
      return false;
    }

    return requestBindings.stream().anyMatch(parameterBinding -> isSameBinding(parameter, parameterBinding));
  }

  private Optional<AuxiliarParameterBindingBuilder> getBodyBinding(List<AuxiliarParameterBindingBuilder> requestBindings) {
    if (requestBindings == null) {
      return Optional.empty();
    }

    return requestBindings.stream().filter(this::isBodyBinding).findFirst();
  }

  private boolean isSameBinding(ParameterBuilder parameter, AuxiliarParameterBindingBuilder parameterBinding) {
    return parameter.getParameterType().equals(parameterBinding.getParameterType())
        && parameter.getExternalName().equals(parameterBinding.getName());
  }

  private boolean isBodyBinding(AuxiliarParameterBindingBuilder parameterBinding) {
    return parameterBinding.getParameterType().equals(BODY);
  }

  private List<AuxiliarParameterBindingBuilder> buildResponseBindings(AuxiliarParameterBindingsDescriptor parameterBindings) {
    return buildRequestBindings(parameterBindings);
  }

  private List<AuxiliarParameterBindingBuilder> buildRequestBindings(AuxiliarParameterBindingsDescriptor parameterBindings) {
    if (parameterBindings == null) {
      return null;
    }

    final List<AuxiliarParameterBindingBuilder> parameterBindingList = new LinkedList<>();

    if (parameterBindings.getUriParameters() != null) {
      for (AuxiliarParameterBindingDescriptor uriParam : parameterBindings.getUriParameters()) {
        parameterBindingList.add(buildParameterBinding(uriParam, URI));
      }
    }

    if (parameterBindings.getQueryParameters() != null) {
      for (AuxiliarParameterBindingDescriptor queryParam : parameterBindings.getQueryParameters()) {
        parameterBindingList.add(buildParameterBinding(queryParam, QUERY));
      }
    }

    if (parameterBindings.getHeaders() != null) {
      for (AuxiliarParameterBindingDescriptor header : parameterBindings.getHeaders()) {
        parameterBindingList.add(buildParameterBinding(header, HEADER));
      }
    }

    if (parameterBindings.getBodyBindings() != null) {
      for (AuxiliarParameterBindingDescriptor body : parameterBindings.getBodyBindings()) {
        parameterBindingList.add(buildParameterBinding(body, BODY));
      }
    }

    final String requestBodyExpression = parameterBindings.getRequestBodyExpression();
    if (requestBodyExpression != null) {
      parameterBindingList.add(buildBodyParameterBinding(requestBodyExpression));
    }

    return parameterBindingList;
  }

  private AuxiliarParameterBindingBuilder buildBodyParameterBinding(String value) {
    AuxiliarParameterBindingBuilder auxiliarParameterBindingBuilder = new AuxiliarParameterBindingBuilder(BODY.getName(), BODY);
    auxiliarParameterBindingBuilder.value(value);
    return auxiliarParameterBindingBuilder;
  }

  private AuxiliarParameterBindingBuilder buildParameterBinding(AuxiliarParameterBindingDescriptor descriptor,
                                                                ParameterType parameterType) {
    AuxiliarParameterBindingBuilder auxiliarParameterBindingBuilder =
        new AuxiliarParameterBindingBuilder(descriptor.getName(), parameterType);
    auxiliarParameterBindingBuilder.value(descriptor.getValue());

    final String schemaType = descriptor.getInputType();
    if (StringUtils.isNotBlank(schemaType)) {
      final TypeDefinitionBuilder typeDefinitionBuilder = auxiliarParameterBindingBuilder.getTypeDefinitionBuilder();
      typeDefinitionBuilder.object().typeSchema(descriptor::getInputType);
      typeDefinitionBuilder.mediaType(loadMediaType(descriptor.getContentType()));
    }

    return auxiliarParameterBindingBuilder;
  }

  private List<AuxiliarParameterBuilder> buildParameters(List<AuxiliarParameterDescriptor> parameterDescriptors) {
    final List<AuxiliarParameterBuilder> parameters = new LinkedList<>();
    if (parameterDescriptors != null) {
      for (AuxiliarParameterDescriptor parameterDescriptor : parameterDescriptors) {
        parameters.add(buildParameter(parameterDescriptor));
      }
    }

    return parameters;
  }

  private AuxiliarParameterBuilder buildParameter(AuxiliarParameterDescriptor parameterDescriptor) {
    AuxiliarParameterBuilder auxiliarParameterBuilder = new AuxiliarParameterBuilder(parameterDescriptor.getName());
    auxiliarParameterBuilder
        .displayName(parameterDescriptor.getDisplayName())
        .description(parameterDescriptor.getDescription())
        .required(parameterDescriptor.isRequired());

    final DataTypeDescriptor dataType = parameterDescriptor.getDataType();
    if (dataType != null) {
      final ParameterDataType parameterDataType = ParameterDataType.forName(dataType.getName());
      auxiliarParameterBuilder.dataType(parameterDataType);
    } else {
      final TypeDefinitionBuilder typeDefinitionBuilder = auxiliarParameterBuilder.getTypeDefinitionBuilder();
      if (StringUtils.isNotBlank(parameterDescriptor.getInputType())) {
        typeDefinitionBuilder.object().typeSchema(parameterDescriptor::getInputType);
      }
      typeDefinitionBuilder.mediaType(loadMediaType(parameterDescriptor.getContentType()));
    }

    return auxiliarParameterBuilder;
  }

  private static MediaType loadMediaType(String contentType) {
    return isEmpty(contentType) ? null : MediaType.valueOf(contentType);
  }

}
